
local MakePlayerCharacter = require "prefabs/player_common"


local assets = {

        Asset( "ANIM", "anim/player_basic.zip" ),
        Asset( "ANIM", "anim/player_idles_shiver.zip" ),
        Asset( "ANIM", "anim/player_actions.zip" ),
        Asset( "ANIM", "anim/player_actions_axe.zip" ),
        Asset( "ANIM", "anim/player_actions_pickaxe.zip" ),
        Asset( "ANIM", "anim/player_actions_shovel.zip" ),
        Asset( "ANIM", "anim/player_actions_blowdart.zip" ),
        Asset( "ANIM", "anim/player_actions_eat.zip" ),
        Asset( "ANIM", "anim/player_actions_item.zip" ),
        Asset( "ANIM", "anim/player_actions_uniqueitem.zip" ),
        Asset( "ANIM", "anim/player_actions_bugnet.zip" ),
        Asset( "ANIM", "anim/player_actions_fishing.zip" ),
        Asset( "ANIM", "anim/player_actions_boomerang.zip" ),
        Asset( "ANIM", "anim/player_bush_hat.zip" ),
        Asset( "ANIM", "anim/player_attacks.zip" ),
        Asset( "ANIM", "anim/player_idles.zip" ),
        Asset( "ANIM", "anim/player_rebirth.zip" ),
        Asset( "ANIM", "anim/player_jump.zip" ),
        Asset( "ANIM", "anim/player_amulet_resurrect.zip" ),
        Asset( "ANIM", "anim/player_teleport.zip" ),
        Asset( "ANIM", "anim/wilson_fx.zip" ),
        Asset( "ANIM", "anim/player_one_man_band.zip" ),
        Asset( "ANIM", "anim/shadow_hands.zip" ),
        Asset( "SOUND", "sound/sfx.fsb" ),
        Asset( "SOUND", "sound/wilson.fsb" ),
        Asset( "ANIM", "anim/beard.zip" ),

        Asset( "ANIM", "anim/wogmaw.zip" ),
        Asset( "ANIM", "anim/ghost_wogmaw_build.zip" ),
		
		Asset( "IMAGE", "images/inventoryimages.tex" ),
		Asset( "ATLAS", "images/inventoryimages.xml" ),
}
local prefabs = {

}

-- Custom starting items
local start_inv = {

	"longsword",
	"healingsalve",
	"healingsalve",
	"healingsalve",

}

-- When the character is revived from human
local function onbecamehuman(inst)
	
	-- Set speed when loading or reviving from ghost (optional)
	inst.components.locomotor.walkspeed = 4
	inst.components.locomotor.runspeed = 6
	
	inst.Light:SetFalloff(1)
    inst.Light:SetIntensity(0.2)
    inst.Light:SetRadius(1)
    inst.Light:SetColour(97/255, 146/255, 17/255)
    inst.Light:Enable(false)
	
	inst.components.hunger.hungerrate = 3.5 * TUNING.WILSON_HUNGER_RATE
	
	inst.components.talker:Say("Time to feast!")
	
end

-- When loading or spawning the character
local function onload(inst)
    inst:ListenForEvent("ms_respawnedfromghost", onbecamehuman)
	
	if not inst:HasTag("playerghost") then
		onbecamehuman(inst)
    end
	
end

local function HungerFlux(inst, data)
    local hunger = data.newpercent
	local health_percent = inst.components.health:GetPercent()
	local hp = inst.components.health.maxhealth
    if 1 == hunger then
		if inst.components.health.maxhealth <= 200 then
			inst.components.health.maxhealth = math.ceil(hp + 7)
			inst.components.health:SetPercent(health_percent)
			inst.components.hunger:SetPercent(0.2)
			inst.components.hunger.hungerrate = 3.5 * TUNING.WILSON_HUNGER_RATE
			inst.Light:SetIntensity(0.2)
		else 
			inst.components.hunger:SetPercent(0.99)
		end
	elseif 0.8 < hunger then
        inst.components.hunger.hungerrate = 10 * TUNING.WILSON_HUNGER_RATE
		inst.Light:SetIntensity(0.8)
        inst.Light:Enable(true)
    elseif 0.6 < hunger and hunger <= 0.8 then
        inst.components.hunger.hungerrate = 8 * TUNING.WILSON_HUNGER_RATE
		inst.Light:SetIntensity(0.2)
        inst.Light:Enable(true)
    elseif 0.4 < hunger and hunger <= 0.6 then
        inst.components.hunger.hungerrate = 6 * TUNING.WILSON_HUNGER_RATE
		inst.Light:SetIntensity(0.2)
        inst.Light:Enable(false)
    elseif 0.2 < hunger and hunger <= 0.4 then
        inst.components.hunger.hungerrate = 5 * TUNING.WILSON_HUNGER_RATE
		inst.Light:SetIntensity(0.2)
        inst.Light:Enable(false)
    elseif hunger <= 0.2 then
        inst.components.hunger.hungerrate = 3.5 * TUNING.WILSON_HUNGER_RATE
        inst.Light:Enable(false)
    end
	
end

local function rangedattack(inst,data)

	local equip = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
	local dmg = 25
	local projectile = nil
	
	if not inst.sg:HasStateTag("attack") or not data.target then 
		return 
	end
	
	if data.target:HasTag("bird") and data.target.sg:HasStateTag("flying") then
		return
	end
	
	if data.target.prefab == "buzzard" and data.target.sg:HasStateTag("flying") then
		return
	end
	
	if data.target.prefab == "mole" and data.target.isunder then
		return
	end
	
	if data.target.prefab == "birchnutdrake" and data.target.sg:HasStateTag("exit") then
		return
	end
	
	if equip and equip.components and not equip.components.projectile then
		projectile = SpawnPrefab("spit")
	
		if not projectile.components.weapon then
			projectile:AddComponent("weapon")
		end 
	
		if equip and equip.components and equip.components.weapon then
			dmg = equip.components.weapon.damage
			if dmg >= 40 then
				dmg = math.ceil(((equip.components.weapon.damage - 40)*0.5)+40)
			end
			projectile.components.weapon:SetDamage(dmg)
		end
	
		projectile.Transform:SetPosition(inst.Transform:GetWorldPosition())
		projectile.components.projectile:Throw(inst, data.target)
	
		if equip and equip.components and equip.components.finiteuses then
			equip.components.finiteuses:Use()
		end
		
	else if not equip then
		projectile = SpawnPrefab("spit")
	
		if not projectile.components.weapon then
			projectile:AddComponent("weapon")
		end 
		
		projectile.components.weapon:SetDamage(dmg)
		projectile.Transform:SetPosition(inst.Transform:GetWorldPosition())
		projectile.components.projectile:Throw(inst, data.target)
		end
	end
	
end

local function equipchange(inst, data)
    local equiphand = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
    if equiphand and equiphand.prefab == "hambat" then
inst:DoTaskInTime(0,function() 
					inst:PushEvent("unequip",inst)
					equiphand:Remove()
					end) 
inst:DoTaskInTime(0.75,function() 
					local food = SpawnPrefab("meat")
					local buffaction = BufferedAction(inst, food, ACTIONS.EAT)
					inst:PushBufferedAction(buffaction)
					end) 
inst:DoTaskInTime(3,function() 
					inst.components.talker:Say("Meat on stick!")
					end)
    end
end

-- This initializes for both the server and client. Tags can be added here.
local common_postinit = function(inst) 
	-- Minimap icon
	inst.MiniMapEntity:SetIcon( "wogmaw.tex" )
end

local function wogmawfood(inst, food)
	if inst.components.eater and food.components.edible.foodtype == "VEGGIE" then
		inst.components.talker:Say("Want meat!")
	elseif inst.components.eater and food:HasTag("rawmeat") then
		inst.components.sanity:DoDelta(12)
	elseif food.prefab == "eel" or food.prefab == "fish" then
		inst.components.sanity:DoDelta(2)
	end
end



-- This initializes for the server only. Components are added here.
local master_postinit = function(inst)
	-- choose which sounds this character will play
	inst.soundsname = "webber"
	
	-- Uncomment if "wathgrithr"(Wigfrid) or "webber" voice is used
    inst.talker_path_override = "dontstarve_DLC001/characters/"
	
	-- Stats	
	inst.components.health:SetMaxHealth(75)
	inst.components.hunger:SetMax(1000)
	inst.components.sanity:SetMax(125)
	
	inst:DoTaskInTime(0.2, function()
		inst.components.hunger:SetPercent(0.2)
		inst.components.talker:Say("Want food!")
	end)
	
	inst.components.eater.ignoresspoilage = true
	inst.components.eater.strongstomach = true
	inst.components.eater:SetCanEatRaw()
	inst.components.eater:SetCanEatHorrible()
	
	inst.components.eater:SetOnEatFn(wogmawfood)
	
	-- Damage multiplier (optional)
    inst.components.combat.damagemultiplier = 1
	inst.components.combat:SetAttackPeriod(.75)
	inst.components.combat:SetDefaultDamage(10)
	inst.components.combat:SetRange(6, 1)
	
	-- Hunger rate (optional)
	inst.components.hunger.hungerrate = 3.5 * TUNING.WILSON_HUNGER_RATE
	
	inst:ListenForEvent("hungerdelta", HungerFlux)
	
	inst:ListenForEvent("onmissother", rangedattack)
	
	inst:ListenForEvent("equip", equipchange)
	
	inst.components.sanity.neg_aura_mult = 3
	
	inst.Light:SetFalloff(1)
    inst.Light:SetIntensity(0.2)
    inst.Light:SetRadius(1)
    inst.Light:SetColour(97/255, 146/255, 17/255)
    inst.Light:Enable(false)
	
	TUNING.KRAMPUS_NAUGHTINESS_DECAY_PERIOD = TUNING.KRAMPUS_NAUGHTINESS_DECAY_PERIOD * 0.20
	
	inst.OnLoad = onload
    inst.OnNewSpawn = onload
	
	
end

return MakePlayerCharacter("wogmaw", prefabs, assets, common_postinit, master_postinit, start_inv)